/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui;

import eu.svjatoslav.sixth.e3d.geometry.Box;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.geometry.Transform;
import eu.svjatoslav.sixth.e3d.gui.humaninput.MouseInteractionController;
import eu.svjatoslav.sixth.e3d.gui.humaninput.UserInputHandler;
import eu.svjatoslav.sixth.e3d.gui.textEditorComponent.KeyboardHelper;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.wireframe.WireframeBox;

import java.awt.event.KeyEvent;

public class GuiComponent extends AbstractCompositeShape implements
        UserInputHandler, MouseInteractionController {

    private static final String GROUP_GUI_FOCUS = "gui.focus";
    public final ViewContext viewContext;
    Box containingBox = new Box();
    private WireframeBox borders = null;

    private boolean borderShown = false;

    public GuiComponent(final Transform transform,
                        final ViewContext viewContext, final Point3D size) {
        super(transform);
        this.viewContext = viewContext;
        setDimensions(size);
    }

    @Override
    public boolean beforeViewUpdate(final ViewContext viewContext,
                                    final int millisecondsSinceLastFrame) {
        return false;
    }

    private WireframeBox createBorder() {
        final LineAppearance appearance = new LineAppearance(10,
                new eu.svjatoslav.sixth.e3d.renderer.raster.Color(255, 0, 0, 100));

        final double borderSize = 10;

        final Box borderArea = containingBox.clone().addBorder(borderSize);

        return new WireframeBox(borderArea, appearance);
    }

    @Override
    public void focusLost(final ViewContext viewContext) {
        hideBorder();
    }

    @Override
    public void focusReceived(final ViewContext viewContext) {
        showBorder();
    }

    public WireframeBox getBorders() {
        if (borders == null)
            borders = createBorder();
        return borders;
    }

    public int getDepth() {
        return (int) containingBox.getDepth();
    }

    public int getHeight() {
        return (int) containingBox.getHeight();
    }

    public int getWidth() {
        return (int) containingBox.getWidth();
    }

    public void hideBorder() {
        if (!borderShown)
            return;
        borderShown = false;
        removeGroup(GROUP_GUI_FOCUS);
    }

    @Override
    public void keyPressed(final KeyEvent event, final ViewContext viewContext) {
        if (event.getKeyChar() == KeyboardHelper.ESC)
            viewContext.getKeyboardFocusTracker().popFocusOwner();
    }

    @Override
    public void keyReleased(final KeyEvent event, final ViewContext viewContext) {
    }

    @Override
    public void mouseClicked() {
        viewContext.getKeyboardFocusTracker().setFocusOwner(this);
    }

    @Override
    public void mouseEntered() {
    }

    @Override
    public void mouseExited() {
    }

    private void setDimensions(final Point3D size) {
        containingBox.setSizeCentered(size);
    }

    public void showBorder() {
        if (borderShown)
            return;
        borderShown = true;
        addShape(getBorders(), GROUP_GUI_FOCUS);
    }

}
