/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster;

import eu.svjatoslav.sixth.e3d.gui.RenderingContext;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.AbstractCoordinateShape;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

public class RenderAggregator {

    ArrayList<AbstractCoordinateShape> shapes = new ArrayList<>();
    ShapesComparator comparator = new ShapesComparator();

    public void paint(final RenderingContext renderBuffer) {

        Collections.sort(shapes, comparator);

        for (final AbstractCoordinateShape shape : shapes)
            shape.paint(renderBuffer);

    }

    public void queueShapeForRendering(final AbstractCoordinateShape shape) {
        shapes.add(shape);
    }

    public void reset() {
        shapes.clear();
    }

    static class ShapesComparator implements Comparator<AbstractCoordinateShape>, Serializable {

        @Override
        public int compare(final AbstractCoordinateShape o1, final AbstractCoordinateShape o2) {
            if (o1.getZ() < o2.getZ())
                return 1;
            else if (o1.getZ() > o2.getZ())
                return -1;

            return Integer.compare(o1.shapeId, o2.shapeId);
        }
    }
}
